@@ -1,14 +1,24 @@
-#' Compute the Empirical Allan Variance
+#' @title Compute the Empirical Allan Variance
 #'
+#' @description
 #' This function estimates the Allan variance.
 #' @param x     A \code{vec} of time series observations or an \code{imu} object.
 #' @param type  A \code{string} containing either \code{"mo"} for Maximal Overlap or \code{"to"} for Tau Overlap.
-#' @param freq  An \code{integer} with the frequency at which the error signal is measured.
-#' @return  A \code{list} that contains:
+#' @param ...   Further arguments passed to other methods.
+#' @return  If the input \code{x} is a \code{vec}, then the function returns a \code{list} that contains:
 #' \itemize{
-#'  \item{"levels": }{The averaging time at each level.}
-#'  \item{"allan": }{The estimated Allan variance.}
-#'  \item{"type": }{Type of estimator (\code{mo} or \code{to}).}
+#'  \item "levels": The averaging time at each level.
+#'  \item "allan": The estimated Allan variance.
+#'  \item "type": Type of estimator (\code{mo} or \code{to}).
+#' }
+#' If the input \code{x} is an \code{imu} object, then the function returns a \code{list} that contains:
+#' \itemize{
+#'  \item "sensor": Name of the sensor.
+#'  \item "freq": The frequency at which the error signal is measured.
+#'  \item "n": Sample size of the data.
+#'  \item "type": The types of sensors considered in the data.
+#'  \item "axis": The axes of sensors considered in the data.
+#'  \item "avar": A list containing the computed Allan variance based on the data.
 #' }
 #' @details
 #' The decomposition and the amount of time it takes to perform this function depends on whether you are using
@@ -34,13 +44,21 @@
 #' where \eqn{ {{\bar y}_t}\left( \tau  \right) = \frac{1}{\tau }\sum\limits_{i = 0}^{\tau  - 1} {{{\bar y}_{t - i}}} }{See PDF Manual}.
 #'
 #' @references Long-Memory Processes, the Allan Variance and Wavelets, D. B. Percival and P. Guttorp
+#' @rdname avar
+#' @export
 #' @examples
 #' set.seed(999)
 #' Xt = rnorm(10000)
 #' av_mat_mo = avar(Xt, type = "mo", freq = 100)
 #' av_mat_tau = avar(Xt, type = "to")
 #'
-avar = function(x, type = "mo", freq = 1) {
+avar = function(x, type = "mo", ...){
+  UseMethod("avar")
+}
+
+#' @rdname avar
+#' @export
+avar.default = function(x, type = "mo", freq = 1, ...) {
 
   if(is.null(x) | length(x) <=1 | dim(as.matrix(x))[2] >1){
     stop("Provide a vector or an 'imu' object")
@@ -62,15 +80,80 @@ avar = function(x, type = "mo", freq = 1) {
   av$lci = av$adev - 2*av$errors*av$adev
   av$uci = av$adev + 2*av$errors*av$adev
   av$type = type
+  av$freq = freq
   av$n = length(x)
   class(av) = c("avar", "list")
   av
 }
 
+#' @rdname avar
+#' @export
+avar.imu = function(x, type = "mo", ...){
+  # Retrive sensor name
+  if (!is.null(attr(x, "stype"))){
+    sensor_name = attr(x, "stype")
+  }else{
+    warning("Unknown sensor name. IMU object is missing some information.")
+    sensor_name = NULL
+  }
 
-#' Prints Allan Variance
+  # Retrive freq
+  if (!is.null(attr(x, "freq"))){
+    freq = attr(x, "freq")
+  }else{
+    warning("Unknown frequency. IMU object is missing some information. Freq is set to 1 by default.")
+    freq = 1
+  }
+
+  # Retrive sample size
+  if (!is.null(attr(x, "dim"))){
+    n = attr(x, "dim")[1]
+  }else{
+    warning("Unknown sample size. IMU object is missing some information.")
+    n = NULL
+  }
+
+  # Retrive col names
+  if (!is.null(attr(x, "dimnames")[[2]])){
+    col_names = attr(x, "dimnames")[[2]]
+  }else{
+    stop("Unknown colunms names. IMU object is missing some information.")
+    col_names = NULL
+  }
+
+  # Retrive sensor
+  if (!is.null(attr(x, "sensor"))){
+    sensor = attr(x, "sensor")
+  }else{
+    warning("Unknown sensor. IMU object is missing some information.")
+    sensor = NULL
+  }
+
+  # Retrive axis
+  if (!is.null(attr(x, "axis"))){
+    ax = attr(x, "axis")
+  }else{
+    warning("Unknown axes. IMU object is missing some information.")
+    ax = NULL
+  }
+
+  # Compute avar
+  m = length(col_names)
+  av = list()
+  for (i in 1:m){
+    av[[i]] = avar.default(x[,i], type = type, freq = freq)
+  }
+  names(av) = col_names
+  out = list(sensor = sensor_name, freq = freq, n = n, type = sensor, axis = ax, avar = av)
+  class(out) = "imu_avar"
+  invisible(out)
+}
+
+
+
+#' @title Prints Allan Variance
 #'
-#' Displays the information on the output of the `avar()` function
+#' @description Displays the information on the output of the `avar()` function
 #' @method print avar
 #' @export
 #' @param x   A \code{avar} object.
@@ -91,20 +174,20 @@ print.avar = function(x, ...) {
   print(x$type, digits=5)
 }
 
-#' Summary Allan Variance
+#' @title Summary Allan Variance
 #'
-#' Displays the summary table of the output of the `avar()` function
+#' @description Displays the summary table of the output of the `avar()` function
 #' @method summary avar
 #' @export
 #' @param object A \code{avar} object.
 #' @param ...    Additional arguments affecting the summary produced.
 #' A \code{table} that contains:
 #' \itemize{
-#'  \item{"Time": }{The averaging time at each level.}
-#'  \item{"AVar": }{The estimated Allan variance.}
-#'  \item{"ADev": }{The estimated Allan deviation.}
-#'  \item{"Lower CI": }{The lower bound of the confidence interval for the Allan deviation (ADev).}
-#'  \item{"Upper CI": }{The upper bound of the confidence interval for the Allan deviation (ADev).}
+#'  \item "Time": The averaging time at each level.
+#'  \item "AVar": The estimated Allan variance.
+#'  \item "ADev": The estimated Allan deviation.
+#'  \item "Lower CI": The lower bound of the confidence interval for the Allan deviation (ADev).
+#'  \item "Upper CI": The upper bound of the confidence interval for the Allan deviation (ADev).
 #' }
 #' @examples
 #' set.seed(999)
@@ -315,8 +398,6 @@ plot.avar = function(x, units = NULL, xlab = NULL, ylab = NULL, main = NULL,
   }
 
   # Add AD
-  lines(x$levels, x$adev, type = "l", col = col_ad, pch = 16)
-
   if (is.null(point_pch)){
     point_pch = 16
   }
@@ -324,210 +405,16 @@ plot.avar = function(x, units = NULL, xlab = NULL, ylab = NULL, main = NULL,
   if (is.null(point_cex)){
     point_cex = 1.25
   }
+  lines(x$levels, x$adev, type = "l", col = col_ad, pch = 16)
   lines(x$levels, x$adev, type = "p", col = col_ad, pch = point_pch, cex = point_cex)
 }
 
-#' @title Computes the Allan Variance Linear Regression estimator
+#' @title Plot Allan Deviation based on IMU Data
 #'
 #' @description
-#' Estimate the parameters of time series models based on the Allan Variance Linear Regression (AVLR) approach
-#' @param x     A \code{vec} of time series observations or an \code{imu} object.
-#' @param qn    A \code{vec} specifying on which scales the parameters of a Quantization Noise (QN) should be computed.
-#' @param wn    A \code{vec} specifying on which scales the parameters of a White Noise (WN) should be computed.
-#' @param rw    A \code{vec} specifying on which scales the parameters of a Random Wakk (RW) should be computed.
-#' @param dr    A \code{vec} specifying on which scales the parameters of a Drift (DR) should be computed.
-#' @param ci    A \code{boolean} to compute parameter confidence intervals.
-#' @param B     A \code{double} for the number of bootstrap replicates to compute the parameter confidence intervals.
-#' @param alpha A \code{double} defining the level of the confidence interval (1 - `alpha`).
-#' @return avlr   A \code{list} that contains:
-#' \itemize{
-#'  \item{"estimates"}{The estimated value of the parameters.}
-#'  \item{"implied_ad"}{The Allan deviation implied by the estimated parameters.}
-#'  \item{"implied_ad_decomp"}{The Allan deviation implied by the estimated parameters for each individual model (if more than one is specified).}
-#'  \item{"av"}{The \code{avar} object computed from the provided data.}
-#' }
-#' @importFrom stats dnorm
-#' @examples
-#' \donttest{
-#' set.seed(999)
-#'
-#' N = 100000
-#' Xt = rnorm(N) + cumsum(rnorm(N, 0, 3e-3))
-#'
-#' av = avar(Xt)
-#' plot(av)
-#'
-#' fit = avlr(Xt, wn = 1:8, rw = 11:15)
-#' fit
-#' plot(fit, decomp = TRUE)
-#' }
-avlr = function(x, qn = NULL, wn = NULL, rw = NULL, dr = NULL,
-                ci = FALSE, B = 100, alpha = 0.05){
-
-  # if (ci == TRUE){
-  #   stop("Method of confidence is not currently supported as it depends on an external package.")
-  # }
-
-  if(is.null(x) | length(x) <=1){
-    stop("Please provide a time series vector or an 'imu' object")
-  }else if(class(x)[1] != "avar"){
-    if(dim(as.matrix(x))[2] >1){
-      stop("Please provide a time series vector or an 'imu' object")
-    } else {
-      x = avar(x)
-    }
-  }
-
-
-  if(sum(sapply(list(qn,wn,rw,dr), is.null)) == 4){
-    stop("Please specify a least one process.")
-  }
-
-  n_processes = 4 - sum(sapply(list(qn,wn,rw,dr), is.null))
-
-  process = rep(NA,n_processes)
-  param = rep(NA,n_processes)
-  implied = matrix(NA,length(x$levels),n_processes)
-
-  counter = 0
-
-  if(!is.null(wn)){
-    if(length(wn) < 1 || !is.whole(wn) || min(wn) < 1 || max(wn) > length(x$allan)){
-      stop("wn incorrectly formatted.")
-    }
-    counter = counter + 1
-    process[counter] = "WN"
-    param[counter] = exp(mean(log(x$adev[wn]) + log(x$levels[wn])/2))
-    implied[,counter] = param[counter]/sqrt(x$levels)
-
-    if (counter == 1){
-      model_estimated = WN(sigma2 = (param[counter])^2)
-    }else{
-      model_estimated = model_estimated + WN(sigma2 = (param[counter])^2)
-    }
-  }
-
-  if(!is.null(qn)){
-    if(length(qn) < 1 || !is.whole(qn) || min(qn) < 1 || max(qn) > length(x$allan)){
-      stop("qn incorrectely formatted.")
-    }
-    counter = counter + 1
-    process[counter] = "QN"
-    param[counter] = (1/sqrt(3))*exp(mean(log(x$adev[qn]) + log(x$levels[qn])))
-    implied[,counter] = sqrt(3)*param[counter]/(x$levels)
-
-    if (counter == 1){
-      model_estimated = QN(q2 = (param[counter])^2)
-    }else{
-      model_estimated = model_estimated + QN(q2 = (param[counter])^2)
-    }
-  }
-
-  if(!is.null(rw)){
-    if(length(rw) < 1 || !is.whole(rw) || min(rw) < 1 || max(rw) > length(x$allan)){
-      stop("rw incorrectely formatted.")
-    }
-    counter = counter + 1
-    process[counter] = "RW"
-    param[counter] = sqrt(3)*exp(mean(log(x$adev[rw]) - log(x$levels[rw])/2))
-    implied[,counter] = param[counter]*sqrt(x$levels/3)
-
-    if (counter == 1){
-      model_estimated = RW(gamma2 = (param[counter])^2)
-    }else{
-      model_estimated = model_estimated + RW(gamma2 = (param[counter])^2)
-    }
-  }
-
-  if(!is.null(dr)){
-    if(length(dr) < 1 || !is.whole(dr) || min(dr) < 1 || max(dr) > length(x$allan)){
-      stop("dr incorrectely formatted.")
-    }
-    counter = counter + 1
-    process[counter] = "DR"
-    param[counter] = sqrt(2)*exp(mean(log(x$adev[dr]) - log(x$levels[dr])))
-    implied[,counter] = param[counter]*x$levels/2
-
-    if (counter == 1){
-      model_estimated = DR(omega = param[counter])
-    }else{
-      model_estimated = model_estimated + DR(omega = param[counter])
-    }
-  }
-
-  implied_ad = apply(implied, 1, sum)
-
-  estimates = t(t(param))
-  rownames(estimates) = process
-  colnames(estimates) = "Value"
-
-  # Bootstrap parameters
-  if (ci == TRUE){
-    out_boot = boostrap_ci_avlr(model = model_estimated,
-                                B = B, n = x$n, qn = qn,
-                                wn = wn, rw = rw, dr = dr,
-                                alpha = alpha)
-    param = 2*param - out_boot$mu
-    out_boot$ci = cbind(param - dnorm(1-alpha/2)*out_boot$sd, param + dnorm(1-alpha/2)*out_boot$sd)
-    print("Parameter estimates corrected for bias via bootstrap")
-  }else{
-    out_boot = NULL
-  }
-
-  x = structure(list(estimates = param,
-                     process_desc = process,
-                     implied_ad = implied_ad,
-                     implied_ad_decomp = implied,
-                     av = x,
-                     model = model_estimated,
-                     ci = out_boot), class = "avlr")
-  invisible(x)
-}
-
-#' Print avlr object
-#'
-#' Displays information about the avlr object
-#' @method print avlr
-#' @export
-#' @keywords internal
-#' @param x   A \code{avlr} object
-#' @param ... Other arguments passed to specific methods
-#' @return Text output via print
-#' @examples
-#' \donttest{
-#' set.seed(999)
-#'
-#' N = 100000
-#' Xt = rnorm(N) + cumsum(rnorm(N, 0, 3e-3))
-#'
-#' fit = avlr(Xt, wn = 1:7, rw = 12:15)
-#' print(fit)
-#' }
-print.avlr = function(x, ...) {
-  if(is.null(x$ci)){
-    cat("\n Estimates: \n")
-    estimates = t(t(x$estimates))
-    rownames(estimates) = x$process_desc
-    colnames(estimates) = "Value"
-    print(estimates)
-  }else{
-    cat("\n Estimates: \n")
-    estimates = t(t(x$estimates))
-    mat = cbind(t(t(x$estimates)),x$ci$ci, x$ci$sd )
-    rownames(mat) = x$process_desc
-    colnames(mat) = c("Value", "CI Low", "CI High", "SD")
-    print(mat)
-  }
-}
-
-#' @title Plot the AVLR with the Allan Deviation
-#'
-#' @description
-#' Displays a plot of the Allan deviation (AD) with the CI values and the AD implied by the estimated parameters.
-#' @method plot avlr
-#' @param x                An \code{avlr} object.
-#' @param decomp           A \code{boolean} that determines whether the contributions of each individual model are plotted.
-#' @param units            A \code{string} that specifies the units of time plotted on the x axis.
+#' Displays a plot of Allan deviation based on IMU data with its corresponding pointwise confidence intervals.
+#' @method plot imu_avar
+#' @param x                An \code{avar} object.
 #' @param xlab             A \code{string} that gives a title for the x axis.
 #' @param ylab             A \code{string} that gives a title for the y axis.
 #' @param main             A \code{string} that gives an overall title for the plot.
@@ -536,77 +423,63 @@ print.avlr = function(x, ...) {
 #' @param ci_ad            A \code{boolean} that determines whether to plot the confidence interval shaded area.
 #' @param nb_ticks_x       An \code{integer} that specifies the maximum number of ticks for the x-axis.
 #' @param nb_ticks_y       An \code{integer} that specifies the maximum number of ticks for the y-axis.
-#' @param legend_position  A \code{string} that specifies the position of the legend (use \code{legend_position = NA} to remove legend).
 #' @param point_pch        A \code{double} that specifies the symbol type to be plotted.
 #' @param point_cex        A \code{double} that specifies the size of each symbol to be plotted.
 #' @param ...              Additional arguments affecting the plot.
-#' @return Plot of Allan deviation and relative confidence intervals for each scale.
-#' @author Stephane Guerrier and Justin Lee
+#' @return A plot of the Allan deviation and relative confidence interval for each scale.
+#' @author Stephane Guerrier and Yuming Zhang
 #' @export
 #' @examples
-#' \donttest{
-#' set.seed(999)
-#'
-#' N = 100000
-#' Xt = rnorm(N) + cumsum(rnorm(N, 0, 3e-3))
-#' av = avlr(Xt, wn = 1:7, rw = 12:15)
-#'
-#' plot.avlr(av)
-#' plot.avlr(av, decomp = TRUE, main = "Simulated white noise", xlab = "Scales")
-#' plot.avlr(av, units = "sec", legend_position = "topright")
-#' plot.avlr(av, col_ad = "darkred", col_ci = "pink")
-#' }
-plot.avlr = function(x, decomp = FALSE,
-                     units = NULL, xlab = NULL, ylab = NULL, main = NULL,
-                     col_ad = NULL, col_ci = NULL, nb_ticks_x = NULL, nb_ticks_y = NULL,
-                     legend_position = NULL, ci_ad = NULL, point_cex = NULL,
-                     point_pch = NULL, ...){
+#' data("navchip_av")
+#' plot(navchip_av)
+plot.imu_avar = function(x, xlab = NULL, ylab = NULL, main = NULL,
+                         col_ad = NULL, col_ci = NULL, nb_ticks_x = NULL, nb_ticks_y = NULL,
+                         ci_ad = NULL, point_pch = NULL, point_cex = NULL, ...){
+  type = unique(x$type)
 
+  if ("Gyroscope" %in% type){
+    gyro_index = which(x$type == "Gyroscope")
+  }else{
+    gyro_index = NULL
+  }
 
-  # Labels
-  if (is.null(xlab)){
-    if (is.null(units)){
-      xlab = expression(paste("Scale ", tau, sep =""))
-    }else{
-      xlab = bquote(paste("Averaging time ", tau, " [", .(units), "]", sep = " "))
+  if ("Accelerometer" %in% type){
+    accel_index = which(x$type == "Accelerometer")
+  }else{
+    accel_index = NULL
+  }
+
+  ncol = length(unique(x$axis))
+  nrow = length(type)
+
+  m = length(x$avar)
+  J = length(x$avar[[1]]$allan)
+
+  # remove negative CI values
+  index_to_remove = c()
+  for (i in 1:m) {
+    if(length(which(x$avar[[i]]$lci<0)) > 0){
+      index_to_remove = c(index_to_remove, which(x$avar[[i]]$lci<0))
     }
   }
-
-  if (is.null(ylab)){
-    ylab = expression(paste("Allan Deviation ", phi, sep = ""))
+  if (!is.null(index_to_remove)){
+    index_to_remove = unique(index_to_remove)
+    index_to_keep = which(seq(1:J) != index_to_remove)
   }else{
-    ylab = ylab
+    index_to_keep = 1:J
   }
 
-  # Main Title
-  if (is.null(main)){
-    main = "Allan Deviation Representation"
+  J = length(index_to_keep)
+  scales = x$avar[[1]]$levels[index_to_keep]
+
+  ci_up = ci_lw = av = matrix(NA, J, m)
+
+  for (i in 1:m){
+    ci_up[,i] = x$avar[[i]]$uci[index_to_keep]
+    ci_lw[,i] = x$avar[[i]]$lci[index_to_keep]
+    av[,i] = x$avar[[i]]$allan[index_to_keep]
   }
 
-  # Line and CI colors
-  if (is.null(col_ad)){
-    col_ad = "darkblue"
-  }
-
-  if (is.null(col_ci)){
-    col_ci = hcl(h = 210, l = 65, c = 100, alpha = 0.2)
-  }
-
-  # Range
-  x_range = range(x$av$levels)
-  if(length(x$av$levels) >= 10){
-    x_low = floor(log10(x_range[1]))
-    x_high = ceiling(log10(x_range[2]))
-  }else{
-    x_low = floor(log2(x_range[1]))
-    x_high = ceiling(log2(x_range[2]))
-  }
-
-
-  y_range = range(cbind(x$av$adev - x$av$adev*x$av$errors, x$av$adev + x$av$adev*x$av$errors))
-  y_low = floor(log10(y_range[1]))
-  y_high = ceiling(log10(y_range[2]))
-
   # Axes
   if (is.null(nb_ticks_x)){
     nb_ticks_x = 6
@@ -616,100 +489,26 @@ plot.avlr = function(x, decomp = FALSE,
     nb_ticks_y = 5
   }
 
+  # Range
+  x_range = range(scales)
+  x_low = floor(log10(x_range[1]))
+  x_high = ceiling(log10(x_range[2]))
+
   x_ticks = seq(x_low, x_high, by = 1)
   if (length(x_ticks) > nb_ticks_x){
     x_ticks = x_low + ceiling((x_high - x_low)/(nb_ticks_x + 1))*(0:nb_ticks_x)
   }
+  x_labels = sapply(x_ticks, function(i) as.expression(bquote(10^ .(i))))
 
-  if(length(x$av$clusters) >= 10){
-    x_labels = sapply(x_ticks, function(i) as.expression(bquote(10^ .(i))))
-  }else{
-    x_labels = sapply(x_ticks, function(i) as.expression(bquote(2^ .(i))))
+
+  # Line and CI colors
+  if (is.null(col_ad)){
+    col_ad = "darkblue"
   }
 
-  y_ticks <- seq(y_low, y_high, by = 1)
-  if (length(y_ticks) > nb_ticks_y){
-    y_ticks = y_low + ceiling((y_high - y_low)/(nb_ticks_y + 1))*(0:nb_ticks_y)
+  if (is.null(col_ci)){
+    col_ci = hcl(h = 210, l = 65, c = 100, alpha = 0.2)
   }
-  y_labels <- sapply(y_ticks, function(i) as.expression(bquote(10^ .(i))))
-
-  # Legend Position
-  if (is.null(legend_position)){
-    #if (which.min(abs(c(y_low, y_high) - log2(x$variance[1]))) == 1){
-    #  legend_position = "topleft"
-    #}else{
-    legend_position = "bottomleft"
-    #}
-  }
-
-  # Main Plot
-  plot(NA, xlim = x_range, ylim = y_range, xlab = xlab, ylab = ylab,
-       log = "xy", xaxt = 'n', yaxt = 'n', bty = "n", ann = FALSE)
-  win_dim = par("usr")
-
-  par(new = TRUE)
-  plot(NA, xlim = x_range, ylim = 10^c(win_dim[3], win_dim[4] + 0.09*(win_dim[4] - win_dim[3])),
-       xlab = xlab, ylab = ylab, log = "xy", xaxt = 'n', yaxt = 'n', bty = "n")
-  win_dim = par("usr")
-
-  # Add Grid
-  if(length(x$av$levels) >=10){
-    abline(v = 10^x_ticks, lty = 1, col = "grey95")
-  }else{
-    abline(v = 2^x_ticks, lty = 1, col = "grey95")
-  }
-  abline(h = 10^y_ticks, lty = 1, col = "grey95")
-
-  # Add Title
-  x_vec = 10^c(win_dim[1], win_dim[2], win_dim[2], win_dim[1])
-  y_vec = 10^c(win_dim[4], win_dim[4],
-               win_dim[4] - 0.09*(win_dim[4] - win_dim[3]),
-               win_dim[4] - 0.09*(win_dim[4] - win_dim[3]))
-  polygon(x_vec, y_vec, col = "grey95", border = NA)
-  text(x = 10^mean(c(win_dim[1], win_dim[2])), y = 10^(win_dim[4] - 0.09/2*(win_dim[4] - win_dim[3])), main)
-
-  # Add Axes and Box
-  lines(x_vec[1:2], rep(10^(win_dim[4] - 0.09*(win_dim[4] - win_dim[3])),2), col = 1)
-  #y_ticks = y_ticks[(2^y_ticks) < 10^(win_dim[4] - 0.09*(win_dim[4] - win_dim[3]))]
-  y_labels = y_labels[1:length(y_ticks)]
-  box()
-  if(length(x$av$levels) >=10){
-    axis(1, at = 10^x_ticks, labels = x_labels, padj = 0.3)
-  }else{
-    axis(1, at = 2^x_ticks, labels = x_labels, padj = 0.3)
-  }
-  axis(2, at = 10^y_ticks, labels = y_labels, padj = -0.2)
-
-  # CI for AD
-  if (ci_ad == TRUE || is.null(ci_ad)){
-    polygon(c(x$av$levels, rev(x$av$levels)), c(x$av$adev - x$av$errors*x$av$adev, rev(x$av$adev + x$av$errors*x$av$adev)),
-            border = NA, col = col_ci)
-  }
-
-  U = dim(x$implied_ad_decomp)[2]
-  col_decomp = hcl(h = seq(100, 375, length = U + 1), l = 65, c = 200, alpha = 1)[1:U]
-
-  # Legend Position
-  if (is.null(legend_position)){
-    #if (which.min(abs(c(y_low, y_high) - log2(x$variance[1]))) == 1){
-    #  legend_position = "topleft"
-    #}else{
-    legend_position = "bottomleft"
-    #}
-  }
-
-  if(decomp == TRUE){
-    # Plot lines of decomp theo
-    for (i in 1:U){
-      lines(x$av$levels, x$implied_ad_decomp[,i], col = col_decomp[i])
-    }
-  }
-  # Plot implied AD
-  lines(t(x$av$levels),x$implied_ad, type = "l", lwd = 3, col = "#F47F24", pch = 1, cex = 1.5)
-  lines(t(x$av$levels),x$implied_ad, type = "p", lwd = 2, col = "#F47F24", pch = 1, cex = 1.5)
-
-  # Add AD
-  lines(x$av$levels, x$av$adev, type = "l", col = col_ad, pch = 16)
 
   if (is.null(point_pch)){
     point_pch = 16
@@ -718,86 +517,124 @@ plot.avlr = function(x, decomp = FALSE,
   if (is.null(point_cex)){
     point_cex = 1.25
   }
-  lines(x$av$levels, x$av$adev, type = "p", col = col_ad, pch = point_pch, cex = point_cex)
 
-  # Add legend
-  CI_conf = .95
-  ad_title_part1 = "Empirical AD "
-
-  if(decomp == TRUE){
-    legend_names = c(as.expression(bquote(paste(.(ad_title_part1), hat(phi)))),
-                     as.expression(bquote(paste("CI(",hat(phi),", ",.(CI_conf),")"))),"Implied AD",
-                     x$process_desc)
-    col_legend = c(col_ad, col_ci,"#F47F24",col_decomp)
-    p_cex_legend = c(1.25, 3, 1.5,rep(NA,U))
-    lty_legend = c(1, NA, rep(1,U))
-    pch_legend = c(16,15,1,rep(NA,U))
-  }else{
-    legend_names = c(as.expression(bquote(paste(.(ad_title_part1), hat(phi)))),
-                     as.expression(bquote(paste("CI(",hat(phi),", ",.(CI_conf),")"))),"Implied AV")
-    col_legend = c(col_ad, col_ci,"#F47F24")
-    p_cex_legend = c(1.25, 3, 1.5)
-    lty_legend = c(1, NA)
-    pch_legend = c(16,15,1)
+  # Main Title
+  if (is.null(main)){
+    main = paste("Allan Variance Representation - ", x$sensor, " @ ", x$freq, " Hz", sep="")
   }
-  if (!is.na(legend_position)){
-    if (legend_position == "topleft"){
-      legend_position = 10^c(1.1*win_dim[1], 0.98*(win_dim[4] - 0.09*(win_dim[4] - win_dim[3])))
-      legend(x = legend_position[1], y = legend_position[2],
-             legend = legend_names, pch = pch_legend, lty = lty_legend,
-             col = col_legend, cex = 1, pt.cex = p_cex_legend, bty = "n")
-    }else{
-      if (legend_position == "topright"){
-        legend_position = 10^c(0.7*win_dim[2], 0.98*(win_dim[4] - 0.09*(win_dim[4] - win_dim[3])))
-        legend(x = legend_position[1], y = legend_position[2],
-               legend =legend_names, pch = pch_legend, lty = lty_legend,
-               col = col_legend, cex = 1, pt.cex = p_cex_legend, bty = "n")
-      }else{
-        legend(legend_position,
-               legend = legend_names, pch = pch_legend, lty = lty_legend,
-               col = col_legend, cex = 1, pt.cex = p_cex_legend, bty = "n")
+
+  # Labels
+  if (is.null(xlab)){
+    xlab = bquote(paste("Averaging time ", tau, " [sec]", sep = " "))
+  }
+
+  if (is.null(ylab)){
+    ylab = expression(paste("Allan Deviation ", phi, sep = ""))
+  }
+
+
+  # Main plot
+  par(omi=rep(1.0, 4), mar=c(0,0,0,0), mfrow=c(nrow,ncol))
+
+  # Gyro
+  if (!is.null(gyro_index)){
+    y_range = c(min(ci_lw[,gyro_index]), max(ci_up[,gyro_index]))
+    y_low = floor(log10(y_range[1]))
+    y_high = ceiling(log10(y_range[2]))
+
+    y_ticks <- seq(y_low, y_high, by = 1)
+    if (length(y_ticks) > nb_ticks_y){
+      y_ticks = y_low + ceiling((y_high - y_low)/(nb_ticks_y + 1))*(0:nb_ticks_y)
+    }
+    y_labels <- sapply(y_ticks, function(i) as.expression(bquote(10^ .(i))))
+
+
+    for (i in seq_along(gyro_index)){
+      plot(NA, xlim = range(scales), ylim = y_range, xaxt="n", yaxt="n", log = "xy", bty = "n")
+      box(col = "grey")
+
+      mtext(paste("Axis - ", x$axis[gyro_index][i], sep = ""), 3, line = 0.5)
+
+      if (i == 1){
+        axis(2, at = 10^y_ticks, labels = y_labels, padj = -0.2, cex = 1.25)
+      }
+
+      if (i == 1){
+        mtext("Gyroscope", 2, line = 4.5)
+        mtext(ylab, 2, line = 2.5)
+      }
+
+      abline(h = 10^y_ticks, col = "grey85")
+      abline(v = 10^x_ticks, col = "grey85")
+
+      # CI for AD
+      if(ci_ad == TRUE || is.null(ci_ad)){
+        polygon(c(scales, rev(scales)), c(ci_lw[,gyro_index[i]], rev(ci_up[,gyro_index[i]])),
+                border = NA, col = col_ci)
+      }
+
+      # Add AD
+      lines(scales, sqrt(av[,gyro_index[i]]), type = "l", col = col_ad, pch = 16)
+      lines(scales, sqrt(av[,gyro_index[i]]), type = "p", col = col_ad, pch = point_pch, cex = point_cex)
+
+      if (is.null(accel_index)){
+        axis(1, at = 10^x_ticks, labels = x_labels, padj = -0.2, cex = 1.25)
       }
     }
   }
-}
+
+  # Accel
+  if (!is.null(accel_index)){
+    y_range = c(min(ci_lw[,accel_index]), max(ci_up[,accel_index]))
+    y_low = floor(log10(y_range[1]))
+    y_high = ceiling(log10(y_range[2]))
+
+    y_ticks <- seq(y_low, y_high, by = 1)
+    if (length(y_ticks) > nb_ticks_y){
+      y_ticks = y_low + ceiling((y_high - y_low)/(nb_ticks_y + 1))*(0:nb_ticks_y)
+    }
+    y_labels <- sapply(y_ticks, function(i) as.expression(bquote(10^ .(i))))
 
 
-#' Compute bootstrap confidence intervals for the AVLR estimator
-#'
-#' @keywords internal
-#' @importFrom stats quantile sd
-#' @param model A \code{ts.model} object that was estimated with the avlr function.
-#' @param B     A \code{double} for the number of bootsrap replicates to compute the confidence intervals.
-#' @param n     A \code{double} with the sample size.
-#' @param qn    A \code{vec} specifying on which scales the parameters of a Quantization Noise (QN) was computed.
-#' @param wn    A \code{vec} specifying on which scales the parameters of a White Noise (WN) was computed.
-#' @param rw    A \code{vec} specifying on which scales the parameters of a Random Wakk (RW) was computed.
-#' @param dr    A \code{vec} specifying on which scales the parameters of a Drift (DR) was computed.
-#' @param alpha A \code{double} defining the level of the confidence interval (1 - `alpha`).
-#' @return   A \code{list} that contains:
-#' \itemize{
-#'  \item{"ci"}{The 1-\code{alpha} confidence intervals.}
-#'  \item{"sd"}{The standard deviation of the estimated parameters.}
-#' }
-boostrap_ci_avlr = function(model, B, n, qn, wn, rw, dr, alpha){
-  results = matrix(NA, B, model$plength)
-  print("Starting bootstrap:")
+    for (i in seq_along(accel_index)){
+      plot(NA, xlim = range(scales), ylim = y_range, xaxt="n", yaxt="n", log = "xy", bty = "n")
+      box(col = "grey")
+      if (i == 1){
+        axis(2, at = 10^y_ticks, labels = y_labels, padj = -0.2, cex = 1.25)
+      }
 
-  for (i in 1:B){
-    x_star = gen_gts(n = n, model = model)
-    results[i, ] = as.numeric(avlr(x_star, qn = qn, wn = wn, rw = rw,
-                                   dr = dr, ci = FALSE)$estimates)
+
+      if (i == 1){
+        mtext("Accelerometer", 2, line = 4.5)
+        mtext(ylab, 2, line = 2.5)
+      }
+
+      if (length(accel_index) == 3 && i == 2){
+        mtext(xlab, 1, line = 3.5)
+      }
+
+      if (is.null(gyro_index)){
+        mtext(paste("Axis - ", x$axis[gyro_index][i], sep = ""), 3, line = 0.5)
+      }
+
+      abline(h = 10^y_ticks, col = "grey85")
+      abline(v = 10^x_ticks, col = "grey85")
+
+      # CI for AD
+      if(ci_ad == TRUE || is.null(ci_ad)){
+        polygon(c(scales, rev(scales)), c(ci_lw[,accel_index[i]], rev(ci_up[,accel_index[i]])),
+                border = NA, col = col_ci)
+      }
+
+      # Add AD
+      lines(scales, sqrt(av[,accel_index[i]]), type = "l", col = col_ad, pch = 16)
+      lines(scales, sqrt(av[,accel_index[i]]), type = "p", col = col_ad, pch = point_pch, cex = point_cex)
+      axis(1, at = 10^x_ticks, labels = x_labels, padj = -0.2, cex = 1.25)
+    }
   }
 
-  mean_parameters = rep(NA, model$plength)
-  ci_parameters = matrix(NA, model$plength, 2)
-  sd_parameters = rep(NA, model$plength)
-
-  for (i in 1:model$plength){
-    mean_parameters[i] = mean(results[,i])
-    ci_parameters[i, ] = as.numeric(quantile(results[,i], probs = c(alpha/2, 1 - alpha/2)))
-    sd_parameters[i] = sd(results[,i])
-  }
-  list(mu = mean_parameters, ci = ci_parameters, sd = sd_parameters)
+  # Add main title
+  mtext(main, side = 3, line = 3, outer = TRUE)
+  par(mfrow = c(1,1))
 }
 
